// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/GardenGroups.hpp>

const std::string GardenGroups::getPuzzleName() const
{
    return "Day 12: Garden Groups";
}

const std::string GardenGroups::getInputFileName() const
{
    return "garden_groups.txt";
}

void GardenGroups::finish()
{
    Grid<bool> isVisited{ lines.size(), lines[0].size() };
    isVisited.fill(false);

    std::stack<Point2> otherRegionsStack{};
    otherRegionsStack.emplace(0, 0);
    while (!otherRegionsStack.empty())
    {
        const Point2 position{ otherRegionsStack.top() };
        otherRegionsStack.pop();
        if (!isVisited.cell(position))
        {
            traverseRegion(isVisited, otherRegionsStack, position);
        }
    }
}

void GardenGroups::traverseRegion(Grid<bool>& isVisited, std::stack<Point2>& otherRegionsStack, const Point2& start)
{
    const char plantType{ getCharAt(start) };
    long long int area{ 0 };
    long long int perimeter{ 0 };
    std::stack<Point2> regionStack{};
    regionStack.push(start);

    while (!regionStack.empty())
    {
        const Point2 position{ regionStack.top() };
        regionStack.pop();
        if (!isVisited.cell(position))
        {
            isVisited.cell(position) = true;
            area++;

            for (const Point2& direction : Point2::cardinalDirections)
            {
                const Point2 next{ position + direction };
                if (isInBounds(next))
                {
                    if (getCharAt(next) == plantType)
                    {
                        if (!isVisited.cell(next))
                        {
                            regionStack.push(next);
                        }
                    }
                    else
                    {
                        perimeter++;
                        if (!isVisited.cell(next))
                        {
                            otherRegionsStack.push(next);
                        }
                    }
                }
                else
                {
                    perimeter++;
                }
            }
        }
    }

    part1 += area * perimeter;
}
